(function (Drupal, Popper) {
  const onceName = 'tooltip';
  const triggerSelector = '[data-tooltip]';
  const tooltipSelector = '.tooltip[data-uuid]';
  const tooltipHoverTimeout = 250;

  /**
   * Enable tooltip elements.
   */
  Drupal.behaviors.tooltip = {
    attach: function (context, settings) {
      /**
       * Generate tooltip from arbitrary content.
       *
       * @param {string} content
       * @param {boolean} displayArrow
       *
       * @returns DOMElement|null
       */
      function createTooltipFromElement(element) {
        let attr = JSON.parse(element.dataset.tooltip || '{}');
        let displayArrow = Boolean(attr.arrow || true);
        let content = attr.content || element.title;
        if (!content.length) {
          return;
        }

        let uniqueId = Date.now() + '-' + Math.random().toString().slice(2, 4);
        let tooltip = document.createElement('div');
        tooltip.classList.add('tooltip');
        tooltip.dataset.uuid = uniqueId;
        tooltip.innerHTML = content;
        if (displayArrow) {
          tooltip.innerHTML += '<div data-popper-arrow></div>';
        }
        document.body.appendChild(tooltip);

        // Set block ID in [data-tooltip] on element in case of Ajax refresh,
        // so tooltip will be instanciated later.
        element.dataset.tooltip = JSON.stringify({ 'block': uniqueId });

        // Prevent overlapping title.
        delete element.title;

        return tooltip;
      };

      /**
       *
       * @param {*} tooltip
       */
      function instanciateTooltip(element, tooltip) {
        let attr = JSON.parse(element.dataset.tooltip || '{}');

        // Otherwise, not displayed correctly is inside [.block__content].
        // @see \Drupal\tooltip\TooltipBlockBase::preRender();
        let arrow = tooltip.querySelector('[data-popper-arrow]');
        tooltip.appendChild(arrow);

        // @todo Use drupalSettings.default_placement
        // @todo Use drupalSettings.default_offset

        // Set attribute to match Popper's documentation.
        // @see https://popper.js.org/docs/v2/tutorial/#arrow
        element.dataset.popperPlacement = attr.placement || 'top';

        // Default offset.
        let options = {};
        options.modifiers = attr.modifiers || [];
        if (!options.modifiers.filter(_modifier => _modifier.name == 'offset')) {
          options.modifiers.push({ name: 'offset', options: { offset: [0, 10] } });
        }

        // Render tooltip.
        const popperInstance = Popper.createPopper(element, tooltip, options);

        // Hide tooltip by default.
        tooltip.classList.add('visually-hidden');

        // Determine if mouse is over tooltip to keep it displayed.
        tooltip.addEventListener('mouseenter', function () {
          this.dataset.reading = true;
        });
        tooltip.addEventListener('mouseleave', function () {
          delete this.dataset.reading;
          this.classList.add('visually-hidden');
        });

        // Show tooltip on focus.
        element.addEventListener('mouseenter', function () {
          tooltip.classList.remove('visually-hidden');
          popperInstance.update();
        });
        element.addEventListener('focus', function () {
          tooltip.classList.remove('visually-hidden');
          popperInstance.update();
        });

        // Hide tooltip on focus out.
        element.addEventListener('mouseleave', function () {
          setTimeout(function () {
            // Keep tooltip alive if mouse on top.
            if (!Boolean(tooltip.dataset.reading)) {
              tooltip.classList.add('visually-hidden');
            }
          }, tooltipHoverTimeout);
        });
        element.addEventListener('blur', function () {
          setTimeout(function () {
            // Keep tooltip alive if mouse on top.
            if (!Boolean(tooltip.dataset.reading)) {
              tooltip.classList.add('visually-hidden');
            }
          }, tooltipHoverTimeout);
        });

        // Disable click on link.
        if (element.tagName == 'A') {
          element.addEventListener('click', function (e) { e.preventDefault(); });
        }
      };

      // Read and instanciate Tooltips.
      once(onceName, triggerSelector, context).forEach(function (element) {
        // Get block if placed on the same page OR create tooltip content element.
        let attr = JSON.parse(element.dataset.tooltip || '{}');
        let tooltipQuerySelector = '.tooltip[data-uuid="' + attr.block + '"]';
        let tooltip = document.querySelector(tooltipQuerySelector);

        // Retrieve tooltip's content dynamically.
        // @see tooltip.routing.yml
        if (!tooltip && Boolean(attr.block)) {
          const ajaxObject = Drupal.ajax({ url: Drupal.url(`tooltip/block/${attr.block}`) });
          ajaxObject.success = function (response, status) {
            element.dataset.tooltip = JSON.stringify({ 'content': response.content });
            tooltip = createTooltipFromElement(element);
            if (tooltip) {
              instanciateTooltip(element, tooltip);
            } else {
              // Remove attributes for a cleaner frontend.
              delete element.dataset.tooltip;
            }
          };
          ajaxObject.execute();
          return;
        }

        if (!tooltip) {
          tooltip = createTooltipFromElement(element);
        }

        if (tooltip) {
          instanciateTooltip(element, tooltip);
        } else {
          // Remove attributes for a cleaner frontend.
          delete element.dataset.tooltip;
        }
      });

      // Hide tooltip by default, if necessary.
      once(onceName, tooltipSelector, context).forEach(function (element) {
        if (!element.classList.contains('visually-hidden')) {
          element.classList.add('visually-hidden');
        }
      });
    },
    detach(context, settings, trigger) {
      if (trigger === 'unload') {
        once.remove(onceName, triggerSelector, context);
        once.remove(onceName, tooltipSelector, context);
      }
    },
  }

})(Drupal, Popper);
